=== modified file 'drizzled/Makefile.am'
--- drizzled/Makefile.am	2008-11-14 23:27:02 +0000
+++ drizzled/Makefile.am	2008-11-15 18:39:51 +0000
@@ -110,6 +110,10 @@
 		ha_statistics.h \
 		handler.h \
 		handler_structs.h \
+		hybrid_type.h \
+		hybrid_type_traits.h \
+		hybrid_type_traits_decimal.h \
+		hybrid_type_traits_integer.h \
 		item_cmpfunc.h \
 		item_create.h \
 		item_func.h \
@@ -191,6 +195,9 @@
 		field_conv.cc \
 		field_iterator.cc \
 		filesort.cc \
+		hybrid_type_traits.cc \
+		hybrid_type_traits_decimal.cc \
+		hybrid_type_traits_integer.cc \
 		init.cc \
 		item_buff.cc \
 		item.cc \

=== modified file 'drizzled/definitions.h'
--- drizzled/definitions.h	2008-11-14 23:27:02 +0000
+++ drizzled/definitions.h	2008-11-15 18:39:51 +0000
@@ -994,5 +994,34 @@
 #define MY_COLL_CMP_CONV              7
 
 
+/*
+  This enum is used to report information about monotonicity of function
+  represented by Item* tree.
+  Monotonicity is defined only for Item* trees that represent table
+  partitioning expressions (i.e. have no subselects/user vars/PS parameters
+  etc etc). An Item* tree is assumed to have the same monotonicity properties
+  as its correspoinding function F:
+
+  [signed] int64_t F(field1, field2, ...) {
+    put values of field_i into table record buffer;
+    return item->val_int(); 
+  }
+
+  NOTE
+  At the moment function monotonicity is not well defined (and so may be
+  incorrect) for Item trees with parameters/return types that are different
+  from INT_RESULT, may be NULL, or are unsigned.
+  It will be possible to address this issue once the related partitioning bugs
+  (BUG#16002, BUG#15447, BUG#13436) are fixed.
+*/
+
+typedef enum monotonicity_info 
+{
+   NON_MONOTONIC,              /* none of the below holds */
+   MONOTONIC_INCREASING,       /* F() is unary and (x < y) => (F(x) <= F(y)) */
+   MONOTONIC_STRICT_INCREASING /* F() is unary and (x < y) => (F(x) <  F(y)) */
+} enum_monotonicity_info;
+
+
 #endif /* DRIZZLE_SERVER_DEFINITIONS_H */
 

=== added file 'drizzled/hybrid_type.h'
--- drizzled/hybrid_type.h	1970-01-01 00:00:00 +0000
+++ drizzled/hybrid_type.h	2008-11-15 18:39:51 +0000
@@ -0,0 +1,64 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef DRIZZLED_HYBRID_TYPE_H
+#define DRIZZLED_HYBRID_TYPE_H
+
+#include <stdint.h>
+#include <drizzled/my_decimal.h>
+
+/*************************************************************************/
+/*
+  A framework to easily handle different return types for hybrid items
+  (hybrid item is an item whose operand can be of any type, e.g. integer,
+  real, decimal).
+*/
+
+class Hybrid_type_traits;
+
+class Hybrid_type
+{
+public:
+  int64_t integer;
+
+  double real;
+  /*
+    Use two decimal buffers interchangeably to speed up += operation
+    which has no native support in decimal library.
+    Hybrid_type+= arg is implemented as dec_buf[1]= dec_buf[0] + arg.
+    The third decimal is used as a handy temporary storage.
+  */
+  my_decimal dec_buf[3];
+  int used_dec_buf_no;
+
+  /*
+    Traits moved to a separate class to
+      a) be able to easily change object traits in runtime
+      b) they work as a differentiator for the union above
+  */
+  const Hybrid_type_traits *traits;
+
+  Hybrid_type() {}
+  /* XXX: add traits->copy() when needed */
+  Hybrid_type(const Hybrid_type &rhs) :traits(rhs.traits) {}
+};
+
+
+
+#endif /* DRIZZLED_HYBRID_TYPE_H */

=== added file 'drizzled/hybrid_type_traits.cc'
--- drizzled/hybrid_type_traits.cc	1970-01-01 00:00:00 +0000
+++ drizzled/hybrid_type_traits.cc	2008-11-15 18:39:51 +0000
@@ -0,0 +1,102 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include <drizzled/server_includes.h>
+#include <drizzled/hybrid_type_traits.h>
+#include CMATH_H
+
+#if defined(CMATH_NAMESPACE)
+using namespace CMATH_NAMESPACE;
+#endif
+
+static const Hybrid_type_traits real_traits_instance;
+
+Item_result Hybrid_type_traits::type() const
+{
+  return REAL_RESULT;
+}
+
+
+void Hybrid_type_traits::fix_length_and_dec(Item *item, Item *arg) const
+{
+  item->decimals= NOT_FIXED_DEC;
+  item->max_length= item->float_length(arg->decimals);
+}
+
+
+/* Hybrid_type operations. */
+void Hybrid_type_traits::set_zero(Hybrid_type *val) const
+{
+  val->real= 0.0;
+}
+
+
+void Hybrid_type_traits::add(Hybrid_type *val, Field *f) const
+{
+  val->real+= f->val_real();
+}
+
+
+void Hybrid_type_traits::div(Hybrid_type *val, uint64_t u) const
+{
+  val->real/= uint64_t2double(u);
+}
+
+
+int64_t Hybrid_type_traits::val_int(Hybrid_type *val,
+                                    bool) const
+{
+  return (int64_t) rint(val->real);
+}
+
+
+double Hybrid_type_traits::val_real(Hybrid_type *val) const
+{
+  return val->real;
+}
+
+
+my_decimal *
+Hybrid_type_traits::val_decimal(Hybrid_type *val, my_decimal *) const
+{
+  double2my_decimal(E_DEC_FATAL_ERROR, val->real, val->dec_buf);
+  return val->dec_buf;
+}
+
+
+String *
+Hybrid_type_traits::val_str(Hybrid_type *val, String *to,
+                            uint8_t decimals) const
+{
+  to->set_real(val->real, decimals, &my_charset_bin);
+  return to;
+}
+
+
+const Hybrid_type_traits *Hybrid_type_traits::instance()
+{
+  return &real_traits_instance;
+}
+
+
+Hybrid_type_traits::Hybrid_type_traits()
+{}
+
+Hybrid_type_traits::~Hybrid_type_traits()
+{}

=== added file 'drizzled/hybrid_type_traits.h'
--- drizzled/hybrid_type_traits.h	1970-01-01 00:00:00 +0000
+++ drizzled/hybrid_type_traits.h	2008-11-15 18:39:51 +0000
@@ -0,0 +1,60 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef DRIZZLED_HYBRID_TYPE_TRAITS_H
+#define DRIZZLED_HYBRID_TYPE_TRAITS_H
+
+#include <stdint.h>
+
+class Item;
+class Field;
+class Hybrid_type;
+class String;
+class my_decimal;
+
+/* Hybryd_type_traits interface + default implementation for REAL_RESULT */
+
+class Hybrid_type_traits
+{
+public:
+
+  virtual Item_result type() const;
+
+  virtual void
+  fix_length_and_dec(Item *item, Item *arg) const;
+
+  /* Hybrid_type operations. */
+  virtual void set_zero(Hybrid_type *val) const;
+  virtual void add(Hybrid_type *val, Field *f) const;
+  virtual void div(Hybrid_type *val, uint64_t u) const;
+
+  virtual int64_t val_int(Hybrid_type *val,
+                          bool unsigned_flag) const;
+  virtual double val_real(Hybrid_type *val) const;
+  virtual my_decimal *val_decimal(Hybrid_type *val, my_decimal *buf) const;
+  virtual String *val_str(Hybrid_type *val, String *buf,
+                          uint8_t decimals) const;
+
+  static const Hybrid_type_traits *instance();
+  Hybrid_type_traits();
+  virtual ~Hybrid_type_traits();
+
+};
+
+#endif /* DRIZZLED_HYBRID_TYPE_TRAITS_H */

=== added file 'drizzled/hybrid_type_traits_decimal.cc'
--- drizzled/hybrid_type_traits_decimal.cc	1970-01-01 00:00:00 +0000
+++ drizzled/hybrid_type_traits_decimal.cc	2008-11-15 18:39:51 +0000
@@ -0,0 +1,116 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include <drizzled/server_includes.h>
+#include <drizzled/definitions.h>
+
+#include <drizzled/hybrid_type_traits_decimal.h>
+
+/* Hybrid_type_traits_decimal */
+static const Hybrid_type_traits_decimal decimal_traits_instance;
+
+
+Item_result Hybrid_type_traits_decimal::type() const { return DECIMAL_RESULT; }
+
+
+void
+Hybrid_type_traits_decimal::fix_length_and_dec(Item *item, Item *arg) const
+{
+  item->decimals= arg->decimals;
+  item->max_length= cmin(arg->max_length + DECIMAL_LONGLONG_DIGITS,
+                        (unsigned int)DECIMAL_MAX_STR_LENGTH);
+}
+
+
+void Hybrid_type_traits_decimal::set_zero(Hybrid_type *val) const
+{
+  my_decimal_set_zero(&val->dec_buf[0]);
+  val->used_dec_buf_no= 0;
+}
+
+
+void Hybrid_type_traits_decimal::add(Hybrid_type *val, Field *f) const
+{
+  my_decimal_add(E_DEC_FATAL_ERROR,
+                 &val->dec_buf[val->used_dec_buf_no ^ 1],
+                 &val->dec_buf[val->used_dec_buf_no],
+                 f->val_decimal(&val->dec_buf[2]));
+  val->used_dec_buf_no^= 1;
+}
+
+
+/**
+  @todo
+  what is '4' for scale?
+*/
+void Hybrid_type_traits_decimal::div(Hybrid_type *val, uint64_t u) const
+{
+  int2my_decimal(E_DEC_FATAL_ERROR, u, true, &val->dec_buf[2]);
+  /* XXX: what is '4' for scale? */
+  my_decimal_div(E_DEC_FATAL_ERROR,
+                 &val->dec_buf[val->used_dec_buf_no ^ 1],
+                 &val->dec_buf[val->used_dec_buf_no],
+                 &val->dec_buf[2], 4);
+  val->used_dec_buf_no^= 1;
+}
+
+
+int64_t
+Hybrid_type_traits_decimal::val_int(Hybrid_type *val, bool unsigned_flag) const
+{
+  int64_t result;
+  my_decimal2int(E_DEC_FATAL_ERROR, &val->dec_buf[val->used_dec_buf_no],
+                 unsigned_flag, &result);
+  return result;
+}
+
+
+double
+Hybrid_type_traits_decimal::val_real(Hybrid_type *val) const
+{
+  my_decimal2double(E_DEC_FATAL_ERROR, &val->dec_buf[val->used_dec_buf_no],
+                    &val->real);
+  return val->real;
+}
+
+
+my_decimal *Hybrid_type_traits_decimal::val_decimal(Hybrid_type *val,
+                                                    my_decimal *) const
+{ return &val->dec_buf[val->used_dec_buf_no]; }
+
+
+String *
+Hybrid_type_traits_decimal::val_str(Hybrid_type *val, String *to,
+                                    uint8_t decimals) const
+{
+  my_decimal_round(E_DEC_FATAL_ERROR, &val->dec_buf[val->used_dec_buf_no],
+                   decimals, false, &val->dec_buf[2]);
+  my_decimal2string(E_DEC_FATAL_ERROR, &val->dec_buf[2], 0, 0, 0, to);
+  return to;
+}
+
+
+const Hybrid_type_traits_decimal *Hybrid_type_traits_decimal::instance()
+{
+  return &decimal_traits_instance;
+}
+
+
+Hybrid_type_traits_decimal::Hybrid_type_traits_decimal()
+{}

=== added file 'drizzled/hybrid_type_traits_decimal.h'
--- drizzled/hybrid_type_traits_decimal.h	1970-01-01 00:00:00 +0000
+++ drizzled/hybrid_type_traits_decimal.h	2008-11-15 18:39:51 +0000
@@ -0,0 +1,56 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef DRIZZLED_HYBRID_TYPE_TRAITS_DECIMAL_H
+#define DRIZZLED_HYBRID_TYPE_TRAITS_DECIMAL_H
+
+#include <stdint.h>
+#include <drizzled/hybrid_type_traits.h>
+
+class Item;
+class Field;
+class Hybrid_type;
+class my_decimal;
+class String;
+
+class Hybrid_type_traits_decimal: public Hybrid_type_traits
+{
+public:
+  virtual Item_result type() const;
+
+  virtual void
+  fix_length_and_dec(Item *arg, Item *item) const;
+
+  /* Hybrid_type operations. */
+  virtual void set_zero(Hybrid_type *val) const;
+  virtual void add(Hybrid_type *val, Field *f) const;
+  virtual void div(Hybrid_type *val, uint64_t u) const;
+
+  virtual int64_t val_int(Hybrid_type *val, bool unsigned_flag) const;
+  virtual double val_real(Hybrid_type *val) const;
+  virtual my_decimal *val_decimal(Hybrid_type *val,
+                                  my_decimal *buf) const;
+  virtual String *val_str(Hybrid_type *val, String *buf,
+                          uint8_t decimals) const;
+  static const Hybrid_type_traits_decimal *instance();
+  Hybrid_type_traits_decimal();
+};
+
+
+#endif /* DRIZZLED_HYBRID_TYPE_TRAITS_DECIMAL_H */

=== added file 'drizzled/hybrid_type_traits_integer.cc'
--- drizzled/hybrid_type_traits_integer.cc	1970-01-01 00:00:00 +0000
+++ drizzled/hybrid_type_traits_integer.cc	2008-11-15 18:39:51 +0000
@@ -0,0 +1,102 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+
+#include <drizzled/server_includes.h>
+#include <drizzled/definitions.h>
+
+#include <drizzled/hybrid_type_traits_integer.h>
+
+
+/* Hybrid_type_traits_integer */
+static const Hybrid_type_traits_integer integer_traits_instance;
+
+
+Item_result Hybrid_type_traits_integer::type() const
+{
+  return INT_RESULT;
+}
+
+
+void
+Hybrid_type_traits_integer::fix_length_and_dec(Item *item, Item *) const
+{
+  item->decimals= 0;
+  item->max_length= MY_INT64_NUM_DECIMAL_DIGITS;
+  item->unsigned_flag= 0;
+}
+
+
+/* Hybrid_type operations. */
+void Hybrid_type_traits_integer::set_zero(Hybrid_type *val) const
+{
+  val->integer= 0;
+}
+
+
+void Hybrid_type_traits_integer::add(Hybrid_type *val, Field *f) const
+{
+  val->integer+= f->val_int();
+}
+
+
+void Hybrid_type_traits_integer::div(Hybrid_type *val, uint64_t u) const
+{
+  val->integer/= (int64_t) u;
+}
+
+
+int64_t Hybrid_type_traits_integer::val_int(Hybrid_type *val, bool) const
+{
+  return val->integer;
+}
+
+
+double Hybrid_type_traits_integer::val_real(Hybrid_type *val) const
+{
+  return (double) val->integer;
+}
+
+
+my_decimal *Hybrid_type_traits_integer::val_decimal(Hybrid_type *val,
+                                                    my_decimal *) const
+{
+  int2my_decimal(E_DEC_FATAL_ERROR, val->integer, 0, &val->dec_buf[2]);
+  return &val->dec_buf[2];
+}
+
+
+String *Hybrid_type_traits_integer::val_str(Hybrid_type *val, String *buf,
+                                            uint8_t) const
+{
+  buf->set(val->integer, &my_charset_bin);
+  return buf;
+}
+
+
+const Hybrid_type_traits_integer *Hybrid_type_traits_integer::instance()
+{
+  return &integer_traits_instance;
+}
+
+
+Hybrid_type_traits_integer::Hybrid_type_traits_integer()
+{}
+
+

=== added file 'drizzled/hybrid_type_traits_integer.h'
--- drizzled/hybrid_type_traits_integer.h	1970-01-01 00:00:00 +0000
+++ drizzled/hybrid_type_traits_integer.h	2008-11-15 18:39:51 +0000
@@ -0,0 +1,55 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef DRIZZLED_HYBRID_TYPE_TRAITS_INTEGER_H
+#define DRIZZLED_HYBRID_TYPE_TRAITS_INTEGER_H
+
+#include <stdint.h>
+#include <drizzled/hybrid_type_traits.h>
+
+class Item;
+class Field;
+class Hybrid_type;
+class my_decimal;
+class String;
+
+class Hybrid_type_traits_integer: public Hybrid_type_traits
+{
+
+public:
+  virtual Item_result type() const;
+
+  virtual void fix_length_and_dec(Item *arg, Item *item) const;
+
+  /* Hybrid_type operations. */
+  virtual void set_zero(Hybrid_type *val) const;
+  virtual void add(Hybrid_type *val, Field *f) const;
+  virtual void div(Hybrid_type *val, uint64_t u) const;
+
+  virtual int64_t val_int(Hybrid_type *val, bool unsigned_flag) const;
+  virtual double val_real(Hybrid_type *val) const;
+  virtual my_decimal *val_decimal(Hybrid_type *val, my_decimal *buf) const;
+  virtual String *val_str(Hybrid_type *val, String *buf,
+                          uint8_t decimals) const;
+
+  static const Hybrid_type_traits_integer *instance();
+  Hybrid_type_traits_integer();
+};
+
+#endif /* HYBRID_TYPE_TRAITS_INTEGER_H */

=== modified file 'drizzled/item.cc'
--- drizzled/item.cc	2008-11-14 23:27:02 +0000
+++ drizzled/item.cc	2008-11-15 18:39:51 +0000
@@ -27,139 +27,6 @@
 
 const String my_null_string("NULL", 4, default_charset_info);
 
-/****************************************************************************/
-
-/* Hybrid_type_traits {_real} */
-
-void Hybrid_type_traits::fix_length_and_dec(Item *item, Item *arg) const
-{
-  item->decimals= NOT_FIXED_DEC;
-  item->max_length= item->float_length(arg->decimals);
-}
-
-static const Hybrid_type_traits real_traits_instance;
-
-const Hybrid_type_traits *Hybrid_type_traits::instance()
-{
-  return &real_traits_instance;
-}
-
-int64_t Hybrid_type_traits::val_int(Hybrid_type *val,
-                                    bool) const
-{
-  return (int64_t) rint(val->real);
-}
-
-my_decimal *
-Hybrid_type_traits::val_decimal(Hybrid_type *val, my_decimal *) const
-{
-  double2my_decimal(E_DEC_FATAL_ERROR, val->real, val->dec_buf);
-  return val->dec_buf;
-}
-
-
-String *
-Hybrid_type_traits::val_str(Hybrid_type *val, String *to, uint8_t decimals) const
-{
-  to->set_real(val->real, decimals, &my_charset_bin);
-  return to;
-}
-
-/* Hybrid_type_traits_decimal */
-static const Hybrid_type_traits_decimal decimal_traits_instance;
-
-const Hybrid_type_traits_decimal *Hybrid_type_traits_decimal::instance()
-{
-  return &decimal_traits_instance;
-}
-
-
-void
-Hybrid_type_traits_decimal::fix_length_and_dec(Item *item, Item *arg) const
-{
-  item->decimals= arg->decimals;
-  item->max_length= cmin(arg->max_length + DECIMAL_LONGLONG_DIGITS,
-                        (unsigned int)DECIMAL_MAX_STR_LENGTH);
-}
-
-
-void Hybrid_type_traits_decimal::set_zero(Hybrid_type *val) const
-{
-  my_decimal_set_zero(&val->dec_buf[0]);
-  val->used_dec_buf_no= 0;
-}
-
-
-void Hybrid_type_traits_decimal::add(Hybrid_type *val, Field *f) const
-{
-  my_decimal_add(E_DEC_FATAL_ERROR,
-                 &val->dec_buf[val->used_dec_buf_no ^ 1],
-                 &val->dec_buf[val->used_dec_buf_no],
-                 f->val_decimal(&val->dec_buf[2]));
-  val->used_dec_buf_no^= 1;
-}
-
-
-/**
-  @todo
-  what is '4' for scale?
-*/
-void Hybrid_type_traits_decimal::div(Hybrid_type *val, uint64_t u) const
-{
-  int2my_decimal(E_DEC_FATAL_ERROR, u, true, &val->dec_buf[2]);
-  /* XXX: what is '4' for scale? */
-  my_decimal_div(E_DEC_FATAL_ERROR,
-                 &val->dec_buf[val->used_dec_buf_no ^ 1],
-                 &val->dec_buf[val->used_dec_buf_no],
-                 &val->dec_buf[2], 4);
-  val->used_dec_buf_no^= 1;
-}
-
-
-int64_t
-Hybrid_type_traits_decimal::val_int(Hybrid_type *val, bool unsigned_flag) const
-{
-  int64_t result;
-  my_decimal2int(E_DEC_FATAL_ERROR, &val->dec_buf[val->used_dec_buf_no],
-                 unsigned_flag, &result);
-  return result;
-}
-
-
-double
-Hybrid_type_traits_decimal::val_real(Hybrid_type *val) const
-{
-  my_decimal2double(E_DEC_FATAL_ERROR, &val->dec_buf[val->used_dec_buf_no],
-                    &val->real);
-  return val->real;
-}
-
-
-String *
-Hybrid_type_traits_decimal::val_str(Hybrid_type *val, String *to,
-                                    uint8_t decimals) const
-{
-  my_decimal_round(E_DEC_FATAL_ERROR, &val->dec_buf[val->used_dec_buf_no],
-                   decimals, false, &val->dec_buf[2]);
-  my_decimal2string(E_DEC_FATAL_ERROR, &val->dec_buf[2], 0, 0, 0, to);
-  return to;
-}
-
-/* Hybrid_type_traits_integer */
-static const Hybrid_type_traits_integer integer_traits_instance;
-
-const Hybrid_type_traits_integer *Hybrid_type_traits_integer::instance()
-{
-  return &integer_traits_instance;
-}
-
-void
-Hybrid_type_traits_integer::fix_length_and_dec(Item *item, Item *) const
-{
-  item->decimals= 0;
-  item->max_length= MY_INT64_NUM_DECIMAL_DIGITS;
-  item->unsigned_flag= 0;
-}
 
 /*****************************************************************************
 ** Item functions

=== modified file 'drizzled/item.h'
--- drizzled/item.h	2008-11-14 23:27:02 +0000
+++ drizzled/item.h	2008-11-15 18:39:51 +0000
@@ -28,126 +28,6 @@
 class Item_field;
 
 
-/*************************************************************************/
-/*
-  A framework to easily handle different return types for hybrid items
-  (hybrid item is an item whose operand can be of any type, e.g. integer,
-  real, decimal).
-*/
-
-struct Hybrid_type_traits;
-
-struct Hybrid_type
-{
-  int64_t integer;
-
-  double real;
-  /*
-    Use two decimal buffers interchangeably to speed up += operation
-    which has no native support in decimal library.
-    Hybrid_type+= arg is implemented as dec_buf[1]= dec_buf[0] + arg.
-    The third decimal is used as a handy temporary storage.
-  */
-  my_decimal dec_buf[3];
-  int used_dec_buf_no;
-
-  /*
-    Traits moved to a separate class to
-      a) be able to easily change object traits in runtime
-      b) they work as a differentiator for the union above
-  */
-  const Hybrid_type_traits *traits;
-
-  Hybrid_type() {}
-  /* XXX: add traits->copy() when needed */
-  Hybrid_type(const Hybrid_type &rhs) :traits(rhs.traits) {}
-};
-
-
-/* Hybryd_type_traits interface + default implementation for REAL_RESULT */
-
-struct Hybrid_type_traits
-{
-  virtual Item_result type() const { return REAL_RESULT; }
-
-  virtual void
-  fix_length_and_dec(Item *item, Item *arg) const;
-
-  /* Hybrid_type operations. */
-  virtual void set_zero(Hybrid_type *val) const { val->real= 0.0; }
-  virtual void add(Hybrid_type *val, Field *f) const
-  { val->real+= f->val_real(); }
-  virtual void div(Hybrid_type *val, uint64_t u) const
-  { val->real/= uint64_t2double(u); }
-
-  virtual int64_t val_int(Hybrid_type *val,
-                          bool unsigned_flag) const;
-  virtual double val_real(Hybrid_type *val) const { return val->real; }
-  virtual my_decimal *val_decimal(Hybrid_type *val, my_decimal *buf) const;
-  virtual String *val_str(Hybrid_type *val, String *buf, uint8_t decimals) const;
-  static const Hybrid_type_traits *instance();
-  Hybrid_type_traits() {}
-  virtual ~Hybrid_type_traits() {}
-};
-
-
-struct Hybrid_type_traits_decimal: public Hybrid_type_traits
-{
-  virtual Item_result type() const { return DECIMAL_RESULT; }
-
-  virtual void
-  fix_length_and_dec(Item *arg, Item *item) const;
-
-  /* Hybrid_type operations. */
-  virtual void set_zero(Hybrid_type *val) const;
-  virtual void add(Hybrid_type *val, Field *f) const;
-  virtual void div(Hybrid_type *val, uint64_t u) const;
-
-  virtual int64_t val_int(Hybrid_type *val, bool unsigned_flag) const;
-  virtual double val_real(Hybrid_type *val) const;
-  virtual my_decimal *val_decimal(Hybrid_type *val,
-                                  my_decimal *buf __attribute__((unused))) const
-  { return &val->dec_buf[val->used_dec_buf_no]; }
-  virtual String *val_str(Hybrid_type *val, String *buf, uint8_t decimals) const;
-  static const Hybrid_type_traits_decimal *instance();
-  Hybrid_type_traits_decimal() {};
-};
-
-
-struct Hybrid_type_traits_integer: public Hybrid_type_traits
-{
-  virtual Item_result type() const { return INT_RESULT; }
-
-  virtual void
-  fix_length_and_dec(Item *arg, Item *item) const;
-
-  /* Hybrid_type operations. */
-  virtual void set_zero(Hybrid_type *val) const
-  { val->integer= 0; }
-  virtual void add(Hybrid_type *val, Field *f) const
-  { val->integer+= f->val_int(); }
-  virtual void div(Hybrid_type *val, uint64_t u) const
-  { val->integer/= (int64_t) u; }
-
-  virtual int64_t val_int(Hybrid_type *val,
-                           bool unsigned_flag __attribute__((unused))) const
-  { return val->integer; }
-  virtual double val_real(Hybrid_type *val) const
-  { return (double) val->integer; }
-  virtual my_decimal *val_decimal(Hybrid_type *val,
-                                  my_decimal *buf __attribute__((unused))) const
-  {
-    int2my_decimal(E_DEC_FATAL_ERROR, val->integer, 0, &val->dec_buf[2]);
-    return &val->dec_buf[2];
-  }
-  virtual String *val_str(Hybrid_type *val, String *buf,
-                          uint8_t decimals __attribute__((unused))) const
-  { buf->set(val->integer, &my_charset_bin); return buf;}
-  static const Hybrid_type_traits_integer *instance();
-  Hybrid_type_traits_integer() {};
-};
-
-
 void dummy_error_processor(Session *session, void *data);
 
 void view_error_processor(Session *session, void *data);
@@ -297,43 +177,14 @@
 };
 
 
-/*
-  This enum is used to report information about monotonicity of function
-  represented by Item* tree.
-  Monotonicity is defined only for Item* trees that represent table
-  partitioning expressions (i.e. have no subselects/user vars/PS parameters
-  etc etc). An Item* tree is assumed to have the same monotonicity properties
-  as its correspoinding function F:
-
-  [signed] int64_t F(field1, field2, ...) {
-    put values of field_i into table record buffer;
-    return item->val_int(); 
-  }
-
-  NOTE
-  At the moment function monotonicity is not well defined (and so may be
-  incorrect) for Item trees with parameters/return types that are different
-  from INT_RESULT, may be NULL, or are unsigned.
-  It will be possible to address this issue once the related partitioning bugs
-  (BUG#16002, BUG#15447, BUG#13436) are fixed.
-*/
-
-typedef enum monotonicity_info 
-{
-   NON_MONOTONIC,              /* none of the below holds */
-   MONOTONIC_INCREASING,       /* F() is unary and (x < y) => (F(x) <= F(y)) */
-   MONOTONIC_STRICT_INCREASING /* F() is unary and (x < y) => (F(x) <  F(y)) */
-} enum_monotonicity_info;
-
 /*************************************************************************/
-typedef bool (Item::*Item_processor) (unsigned char *arg);
 /*
   Analyzer function
     SYNOPSIS
       argp   in/out IN:  Analysis parameter
                     OUT: Parameter to be passed to the transformer
 
-    RETURN 
+     RETURN
       true   Invoke the transformer
       false  Don't do it
 
@@ -341,6 +192,7 @@
 typedef bool (Item::*Item_analyzer) (unsigned char **argp);
 typedef Item* (Item::*Item_transformer) (unsigned char *arg);
 typedef void (*Cond_traverser) (const Item *item, void *arg);
+typedef bool (Item::*Item_processor) (unsigned char *arg);
 
 
 class Item

=== modified file 'drizzled/item_sum.cc'
--- drizzled/item_sum.cc	2008-11-02 03:17:55 +0000
+++ drizzled/item_sum.cc	2008-11-15 18:39:51 +0000
@@ -23,6 +23,11 @@
 #include <drizzled/server_includes.h>
 #include <drizzled/sql_select.h>
 #include <drizzled/error.h>
+#include <drizzled/hybrid_type_traits.h>
+#include <drizzled/hybrid_type_traits_integer.h>
+#include <drizzled/hybrid_type_traits_decimal.h>
+
+
 #include CMATH_H
 
 #if defined(CMATH_NAMESPACE)
@@ -939,6 +944,7 @@
   return &fast_decimal_traits_instance;
 }
 
+
 void Item_sum_distinct::fix_length_and_dec()
 {
   assert(args[0]->fixed);
@@ -980,6 +986,12 @@
 }
 
 
+enum Item_result Item_sum_distinct::result_type () const
+{
+  return val.traits->type();
+}
+
+
 /**
   @todo
   check that the case of CHAR(0) works OK

=== modified file 'drizzled/item_sum.h'
--- drizzled/item_sum.h	2008-11-07 00:15:51 +0000
+++ drizzled/item_sum.h	2008-11-15 18:39:51 +0000
@@ -24,6 +24,7 @@
 
 
 #include <mysys/my_tree.h>
+#include <drizzled/hybrid_type.h>
 
 /*
   Class Item_sum is the base class used for special expressions that SQL calls
@@ -488,7 +489,7 @@
   void update_field() {} // not used
   virtual void no_rows_in_result() {}
   void fix_length_and_dec();
-  enum Item_result result_type () const { return val.traits->type(); }
+  enum Item_result result_type () const;
   virtual void calculate_val_and_count();
   virtual bool unique_walk_function(void *elem);
 };

